# -*- coding: utf-8 -*-
import json

from django.conf import settings
from rest_framework.parsers import ParseError, six

from djangorestframework_camel_case2.settings import api_settings
from djangorestframework_camel_case2.util import underscoreize


class CamelCaseJSONParser(api_settings.PARSER_CLASS):
    def parse(self, stream, media_type=None, parser_context=None):
        parser_context = parser_context or {}
        encoding = parser_context.get('encoding', settings.DEFAULT_CHARSET)

        try:
            data = stream.read().decode(encoding)
            return underscoreize(
                json.loads(data),
                **api_settings.JSON_UNDERSCOREIZE
            )
        except ValueError as exc:
            raise ParseError('JSON parse error - %s' % six.text_type(exc))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys


try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.argv[-1] == 'publish':
    os.system('python setup.py sdist upload')
    sys.exit()

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')
import djangorestframework_camel_case2

setup(
    name='djangorestframework-camel-case2',
    version=djangorestframework_camel_case2.__version__,
    description='Camel case JSON support for Django REST framework.',
    long_description=readme + '\n\n' + history,
    author='Jozef Gáborík',
    author_email='fadawar@gmail.com',
    url='https://github.com/fadawar/djangorestframework-camel-case2',
    packages=[
        'djangorestframework_camel_case2',
    ],
    package_dir={'djangorestframework_camel_case2': 'djangorestframework_camel_case2'},
    include_package_data=True,
    install_requires=[
    ],
    license="BSD",
    zip_safe=False,
    keywords='djangorestframework_camel_case2',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
    ],
    test_suite='tests',
)



import re
from collections import OrderedDict

from django.utils import six

camelize_re = re.compile(r"[a-z0-9]?_[a-z0-9]")


def underscore_to_camel(match):
    group = match.group()
    if len(group) == 3:
        return group[0] + group[2].upper()
    else:
        return group[1].upper()


def camelize(data):
    if isinstance(data, dict):
        new_dict = OrderedDict()
        for key, value in data.items():
            if isinstance(key, six.string_types) and '_' in key:
                new_key = re.sub(camelize_re, underscore_to_camel, key)
            else:
                new_key = key
            new_dict[new_key] = camelize(value)
        return new_dict
    if is_iterable(data) and not isinstance(data, six.string_types):
        return [camelize(item) for item in data]
    return data


def get_underscoreize_re(options):
    if options.get('no_underscore_before_number'):
        pattern = r'([a-z]|[0-9]+[a-z]?|[A-Z]?)([A-Z])'
    else:
        pattern = r'([a-z]|[0-9]+[a-z]?|[A-Z]?)([A-Z0-9])'
    return re.compile(pattern)


def camel_to_underscore(name, **options):
    underscoreize_re = get_underscoreize_re(options)
    return underscoreize_re.sub(r'\1_\2', name).lower()


def underscoreize(data, **options):
    if isinstance(data, dict):
        new_dict = {}
        for key, value in data.items():
            if isinstance(key, six.string_types):
                new_key = camel_to_underscore(key, **options)
            else:
                new_key = key
            new_dict[new_key] = underscoreize(value, **options)
        return new_dict
    if is_iterable(data) and not isinstance(data, six.string_types):
        return [underscoreize(item, **options) for item in data]

    return data


def is_iterable(obj):
    try:
        iter(obj)
    except TypeError:
        return False
    else:
        return True
